<?php

/**
 * HERE Map package for Concrete CMS
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Alex Borisov <linuxoidoz@gmail.com>
 * @copyright 2021, Alex Borisov. All Rights Reserved.
 * @package Concrete\Package\ab_here_map
 */

namespace Concrete\Package\AbHereMap\Block\AbHereMap;

use Concrete\Core\Block\BlockController;
use Concrete\Core\File\File;
use Concrete\Core\File\Type\Type as FileType;
use Concrete\Core\Package\PackageService;

class Controller extends BlockController
{
    
    protected $btWrapperClass = 'ccm-ui';
    protected $btInterfaceWidth = '600';
    protected $btInterfaceHeight = '600';
    protected $btTable = 'btAbHereMap';
    protected $btDefaultSet = 'multimedia';
    
    protected $pin = '';
    
    public function getBlockTypeDescription() 
    {
        return t('Add HERE Map block');
    }

    public function getBlockTypeName() 
    {
        return t('HERE Map');
    }
    
    public function on_start()
    {
        parent::on_start();
        $this->set('app', $this->app);
        
        if (!empty($this->api_key)) {
            $this->requireAsset('here_maps');
            
            $pkg = $this->app->make(PackageService::class)->getByHandle('ab_here_map');
            $this->pin = $pkg->getRelativePath() . '/blocks/ab_here_map/img/pin_blue.svg';
            $this->set('pin', $this->pin);
        }
    }
    
    public function view() 
    {
        $ih = $this->app->make('helper/image');
        $pic = '';
        $img_obj = $this->fid > 0 ? File::getByID($this->fid) : null;
        $img_obj_type = $img_obj ? $img_obj->getTypeObject() : null;
        $img_obj_file_type = $img_obj_type ? $img_obj_type->getGenericType() : null;
        $thumb = $img_obj && $img_obj_file_type === FileType::T_IMAGE ? $ih->getThumbnail($img_obj, 100, 50, false) : null;
        $pic = $thumb ? '<img class="img-fluid" src="' . $thumb->src . '" />' : '';
            
        $data = array(
            'bid' => $this->bID,
            'api_key' => $this->api_key,
            'lang' => $this->lang,
            'lat' => $this->lat,
            'lng' => $this->lng,
            'name' => $this->name,
            'phone' => $this->phone,
            'email' => $this->email,
            'address' => $this->address,
            'desc' => $this->short_desc,
            'pic' => $pic,
            'zoom' => $this->zoom,
            'lock_wheel' => $this->lock_wheel ? true : false,
            'pin' => $this->pin,
        );
        $this->set('data', json_encode($data));
        
        $this->set('show_map', empty($this->api_key) ? false : true);
    }

    public function edit()
    {
        $this->set('languages', ['en-US' => t('English'), 'ru-RU' => t('Russian')]);
    }
    
    public function save($data) 
    {
        $data['api_key'] = $data['api_key'] ?: '';
        $data['lang'] = $data['lang'] ?: '';
        $data['lat'] = $data['lat'] ?: '55.7538';
        $data['lng'] = $data['lng'] ?: '37.6201';
        $data['name'] = $data['name'] ?: '';
        $data['phone'] = $data['phone'] ?: '';
        $data['email'] = $data['email'] ?: '';
        $data['address'] = $data['address'] ?: '';
        $data['short_desc'] = $data['short_desc'] ? trim(strip_tags(html_entity_decode($data['short_desc'], ENT_QUOTES, 'UTF-8'))) : '';
        $data['fid'] = $data['fid'] ?: 0;
        $data['height'] = $data['height'] ?: '400px';
        $data['width'] = $data['width'] ?: '100%';
        $data['zoom'] = $data['zoom'] ?: 12;
        $data['lock_wheel'] = $data['lock_wheel'] ? 1 : 0;
        
        parent::save($data);
    }
     
}
