<?php defined('C5_EXECUTE') or die("Access Denied.");

/**
 * HERE Map package for Concrete CMS
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Alex Borisov <linuxoidoz@gmail.com>
 * @copyright 2021, Alex Borisov. All Rights Reserved.
 * @package Concrete\Package\ab_here_map
 */

use Concrete\Core\File\File;

$al = $app->make('helper/concrete/asset_library');
?>

<style>
.tooltip .tooltip-inner {
    background-color: #666666 !important;
    color: #ffffff !important;
    font-size: 0.9em !important;
    text-align: left !important;
}

.tooltip.bs-tooltip-top .tooltip-arrow::before {
    border-top-color: #666666 !important;
}

.tooltip.bs-tooltip-bottom .tooltip-arrow::before {
    border-bottom-color: #666666 !important;
}

#map { background-color: #eeeeee; }

.ab-here-map-form .hidden {
    display: none !important;
    visibility: hidden !important;
}

</style>

<fieldset class="ab-here-map-form">
    <?php echo $form->hidden('decimals_error', '', ['data-error-title' => t('Oops!'), 'data-error-message' => t('Numbers only')]); ?>

    <div class="mb-3">
        <?php
        echo $form->label('api_key', t('HERE Maps API Key') . '<span class="text-danger fs-6 ms-3 text-opacity-50">' . t('Required') . '</span>');
        echo $form->text('api_key', $api_key, ['maxlength' => 128]);
        echo '<p class="text-muted fs-6">' . t('Max 128 symbols') . '<br />';
        echo t('Obtain API token at ') . '<a href="https://developer.here.com/" target="_blank">developer.here.com</a></p>';
        ?>
    </div>
    
    <?php if (!empty($api_key)) { ?>
        <div class="row mb-3">
            <div class="col-12 col-sm-6">
                <?php
                echo $form->label('lang', t('Select Map Language'));
                echo $form->select('lang', $languages, $lang);
                ?>
            </div>
            <div class="col-12 col-sm-6">
                <?php 
                echo $form->label('fid', t('Image'));
                echo $al->image('ccm-image', 'fid', t('Choose Image'), $fid ? File::getByID($fid) : null);
                ?>
            </div>
        </div>
                
        <div class="row mb-3">
            <div class="col-12 col-sm-6">
                <?php
                echo $form->label('height', t('Map Height') . '<span class="text-danger fs-6 ms-3 text-opacity-50">' . t('Required') . '</span>' . '<i class="fas fa-question-circle ms-1 text-muted launch-tooltip" title="' . h(t('Set map height with \'px\' or \'%\' only')) . '"></i>');
                echo $form->text('height', $height ? $height : '300px', ['maxlength' => 6]);
                echo '<p class="text-muted fs-6">' . t('Max 6 symbols') . '</p>';
                ?>
            </div>
            <div class="col-12 col-sm-6">
                <?php
                echo $form->label('width', t('Map Width') . '<span class="text-danger fs-6 ms-3 text-opacity-50">' . t('Required') . '</span>' . '<i class="fas fa-question-circle ms-1 text-muted launch-tooltip" title="' . h(t('Set map width with \'px\' or \'%\' only')) . '"></i>');
                echo $form->text('width', $width ? $width : '100%', ['maxlength' => 6]);
                echo '<p class="text-muted fs-6">' . t('Max 6 symbols') . '</p>';
                ?>
            </div>
        </div>

        <div class="row mb-3">
            <div class="col-12 col-sm-6">
                <?php
                echo $form->label('zoom', t('Map zoom'));
                echo $form->number('zoom', $zoom ? $zoom : 12, ['min' => '1', 'max' => '50', 'class' => 'form-control decimals']);
                echo '<p class="text-muted fs-6">' . t('Values from 1 to 50') . '</p>';
                ?>
            </div>
            <div class="col-12 col-sm-6">
                <div class="form-check">
                    <?php
                    echo $form->checkbox('lock_wheel', 1, $lock_wheel ? 1 : 0);
                    echo $form->label('lock_wheel', t('Disable mouse wheel zoom over map?'));
                    ?>
                </div>
            </div>
        </div>
        
        <div class="mb-3">
            <?php 
            echo $form->label('name', t('Name') . '<span class="text-danger fs-6 ms-3 text-opacity-50">' . t('Required') . '</span>');
            echo $form->text('name', $name, ['maxlength' => 128]);
            echo '<p class="text-muted fs-6">' . t('Max 128 symbols') . '</p>';
            ?>
        </div>
        
        <div class="row mb-3">
            <div class="col-12 col-sm-6">
                <div class="mb-3">
                    <?php echo $form->label('phone', t('Phone Number')); ?>
                    <div class="input-group">
                        <span class="input-group-text"><i class="fas fa-phone"></i></span>
                        <?php echo $form->text('phone', $phone, ['maxlength' => 20]); ?>
                    </div>
                    <?php echo '<p class="text-muted fs-6">' . t('Max 20 symbols') . '</p>'; ?>
                </div>
            </div>
            <div class="col-12 col-sm-6">
                <div class="mb-3">
                    <?php echo $form->label('email', t('Email')); ?>
                    <div class="input-group">
                        <span class="input-group-text"><i class="fas fa-envelope"></i></span>
                        <?php echo $form->text('email', $email, ['maxlength' => 128]); ?>
                    </div>
                    <?php echo '<p class="text-muted fs-6">' . t('Max 128 symbols') . '</p>'; ?>
                </div>
            </div>
        </div>
            
        <div class="mb-3">
            <?php
            echo $form->label('short_desc', t('Short Description'));
                echo $form->textarea('short_desc', $short_desc, ['rows' => 5]);
                echo '<p class="text-muted fs-6">' . t('Max 1000 symbols') . '</p>';
            ?>
        </div>
        
        <div class="mb-3">
            <?php
            echo $form->label('address', t('Address'));
            echo $form->text('address', $address, ['maxlength' => 255]);
            echo '<p class="text-muted fs-6">' . t('Max 255 symbols') . '</p>';
            ?>
        </div>

        <div class="mb-3">
            <?php echo $form->button('search', t('Search for Locations'), ['class' => 'btn btn-primary']); ?>
            <span id="spinner" class="text-info hidden"><i class="fas fa-spinner fa-spin ms-1"></i></span>
        </div>
        
        <div class="mb-3">
            <?php
            echo $form->label('locations', t('Choose One of Found Locations'));
            echo $form->select('locations', null, null);
            ?>
        </div>
        
        <div class="row mb-3">
            <div class="col-sm-6 col-xs-12">
                <div class="mb-3">
                    <?php
                    echo $form->label('lat', t('Location Latitude') . '<span class="text-danger fs-6 ms-3 text-opacity-50">' . t('Required') . '</span>');
                    echo $form->text('lat', $lat ?: 55.7538, ['maxlength' => 8, 'class' => 'form-control decimals']);
                    echo '<p class="text-muted fs-6">' . t('Max 8 symbols, numeric only') . '</p>';
                    ?>
                </div>
            </div>

            <div class="col-sm-6 col-xs-12">
                <div class="mb-3">
                    <?php
                    echo $form->label('lng', t('Location Longitude') . '<span class="text-danger fs-6 ms-3 text-opacity-50">' . t('Required') . '</span>');
                    echo $form->text('lng', $lng ?: 37.6201, ['maxlength' => 8, 'class' => 'form-control decimals']);
                    echo '<p class="text-muted fs-6">' . t('Max 8 symbols, numeric only') . '</p>';
                    ?>
                </div>
            </div>
        </div>
        
        <div class="my-3"><div id="map" style="height: 300px; width: 100%;"></div></div>
        
    <?php } else { ?>
        <div class="alert alert-info extensions" role="alert">
            <?php
            echo t('After entering the HERE Map API Key, please:%s
                1. save the changes to take effect,%s
                2. refresh the page, e.g. F5%s
                3. edit the block again to set up the map options',
                '<br />', '<br />', '<br />'
            );
            ?>
        </div>
    <?php } ?>    
    
</fieldset>

<script type="text/javascript">
$(document).ready(function() {
    var error_title = $('#decimals_error').attr('data-error-title');
    var error_message = $('#decimals_error').attr('data-error-message');
    var ctrl_key_pressed = false;
    
    $('.decimals').keydown(function (e) {
        if (e.ctrlKey === true || e.metaKey === true) {
            ctrl_key_pressed = true;
        }
        // Allow: ctrl, backspace, delete, tab, escape, enter, - and .
        if ($.inArray(e.keyCode, [17, 8, 46, 9, 27, 13, 109, 173, 110, 190]) !== -1 ||
             // Allow: Ctrl+A, Ctrl+C, Ctrl+V
            (ctrl_key_pressed && (e.keyCode == 65 || e.keyCode == 67 || e.keyCode == 86)) || 
             // Allow: home, end, left, right, down, up
            (e.keyCode >= 35 && e.keyCode <= 40)) {
                 // let it happen, don't do anything
                 ctrl_key_pressed = false;
                 return;
        }
        // Ensure that it is a number and stop the keypress
        if ((e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) && (e.keyCode < 96 || e.keyCode > 105)) {
            e.preventDefault();
            ConcreteAlert.error({
                title: error_title,
                message: error_message,
            });
        }
    });
    
    $('#ccm-form-submit-button').on('click', function(e) {
        let api_key = $.trim($('#api_key').val());
        
        <?php if (!empty($api_key)) { ?>
            let lat = $.trim($('#lat').val());
            let lng = $.trim($('#lng').val());
            let name = $.trim($('#name').val());
            let phone = $.trim($('#phone').val());
            let email = $.trim($('#email').val());
            let address = $.trim($('#address').val());
            let short_desc = $('textarea[name=short_desc]').val();
            let height = $.trim($('#height').val());
            let width = $.trim($('#width').val());
            let regex = /(\d)(px|%)$/;
        <?php } ?>
        
        if (api_key === '' || api_key.length > 128) {
            e.preventDefault();
            e.stopPropagation();
            ConcreteAlert.error({
                title: <?php echo json_encode(t('API Key value error:')); ?>,
                message: <?php echo json_encode(t('API Key cannot be empty and must have max 128 symbols')); ?>
            });
        }
        
        <?php if (!empty($api_key)) { ?>
            else if (name.length < 1 || name.length > 128) {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Name value error:')); ?>,
                    message: <?php echo json_encode(t('Name cannot be empty and must have max 128 symbols')); ?>
                });
            }
            else if (lat === '' || lng === '') {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Coordinates value error:')); ?>,
                    message: <?php echo json_encode(t('Coordinates cannot be empty')); ?>
                });
            }
            else if (phone.length > 20) {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Phone value error:')); ?>,
                    message: <?php echo json_encode(t('Phone must have max 20 symbols')); ?>
                });
            }
            else if (email.length > 1 && (!emailTest(email) || email.length > 128)) {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Email value error:')); ?>,
                    message: <?php echo json_encode(t('Email must be valid and have max 128 symbols')); ?>
                });
            }
            else if (address.length > 255) {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Address value error:')); ?>,
                    message: <?php echo json_encode(t('Address must have max 255 symbols')); ?>
                });
            }
            else if (short_desc.length > 1000) {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Description value error:')); ?>,
                    message: <?php echo json_encode(t('Description must have max 1000 symbols')); ?>
                });
            }
            else if (zoom < 1 || zoom > 50) {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Zoom value error:')); ?>,
                    message: <?php echo json_encode(t('Zoom values can be from 1 to 50')); ?>
                });
            }
            else if (height == '' || !regex.test(height) || width == '' || !regex.test(width)) {
                e.preventDefault();
                e.stopPropagation();
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Map height/width value error:')); ?>,
                    message: <?php echo json_encode(t('Map height/width values cannot be empty and must end with with \'px\' or \'%\'')); ?>
                });
            }
            
            function emailTest(email) {
                let email_test = /^[a-zA-Z0-9.!#$%&'*+\/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/;
                
                if (email_test.test(email)) {
                    return true;
                }
                else {
                    return false;
                }
            }
        <?php } ?>
    });
    
    <?php if (!empty($api_key)) { ?>
        let api_key = $.trim($('#api_key').val());
        let latitude = $('#lat');
        let longitude = $('#lng');
        let spinner = $('#spinner');
        let locations = $('#locations');
        let language = $('#lang').val();
        
        let map, behavior, ui, marker;
        
        let platform = new H.service.Platform({
            'apikey': api_key
        });
        
        let service = platform.getSearchService();
        let defaultLayers = platform.createDefaultLayers({ pois: true, lg: language.substring(0, 2) });
        
        let icon = new H.map.Icon('<?php echo $pin; ?>', {
            size: { w: 40, h: 40 },
        });
        
        let position = new H.geo.Point(55.7538, 37.6201);
        
        lat = latitude.val();
        lng = longitude.val();
        
        if (lat !== '' && lng !== '') {
            position = new H.geo.Point(lat, lng);
        }
        
        setTimeout(function () {
            loadMap(position);
        }, 1000);
    
        $('#search').on('click', function(e) {
            let address = $.trim($('#address').val());
            getLocations(address);
        });

        locations.on('change', function(e) {
            let location = $('#locations option:selected').text();
            geocodeForward(location);
        });
            
        function getLocations(location) {
            if (location != '') {
                spinner.removeClass('hidden');
                
                service.geocode({
                    q: location
                }, (result) => {
                    spinner.addClass('hidden');
                    
                    let items = result.items;
                    
                    if (Array.isArray(items) && items.length > 0) {
                        locations.empty();
                        locations.append($('<option/>').val(0).text('<?php echo t('Select address'); ?>'));
                        
                        items.forEach((item, index) => {
                            locations.append($('<option/>').val(index).text(item.title));
                        });
                    }
                    else {
                        locations.empty();
                        ConcreteAlert.error({
                            title: <?php echo json_encode(t('Geocoding error')); ?>,
                            message: <?php echo json_encode(t('No address found')); ?>,
                        });
                    }
                }, (error) => {
                    spinner.addClass('hidden');
                    ConcreteAlert.error({
                        title: <?php echo json_encode(t('Geocoding failure')); ?>,
                        message: error
                    });
                });
            }
            else {
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Empty Address')); ?>,
                    message: <?php echo json_encode(t('Address must be provided')); ?>
                });
                return;
            }
        }
        
        function geocodeForward(location) {
            if (location != '') {
                service.geocode({
                    q: location,
                    limit: 1
                }, (result) => {
                    spinner.addClass('hidden');
                    
                    let items = result.items;
                    
                    if (Array.isArray(items) && items.length > 0) {
                        let {position} = items[0];
                        let lat = position.lat;
                        let lng = position.lng;
                        
                        latitude.val(lat);
                        longitude.val(lng);
                        setMapPosition(position);
                    }
                    else {
                        locations.empty();
                        ConcreteAlert.error({
                            title: <?php echo json_encode(t('Geocoding error')); ?>,
                            message: <?php echo json_encode(t('No address found')); ?>,
                        });
                    }
                }, (error) => {
                    spinner.addClass('hidden');
                    ConcreteAlert.error({
                        title: <?php echo json_encode(t('Geocoding failure')); ?>,
                        message: error
                    });
                });
            }
            else {
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Empty Address')); ?>,
                    message: <?php echo json_encode(t('Address must be provided')); ?>
                });
                return;
            }
        }
        
        function geocodeReverse(position) {
            service.reverseGeocode({
                at: position.lat.toFixed(4) + ',' + position.lng.toFixed(4),
                limit: '1'
            }, (result) => {
                let items = result.items;
                
                if (Array.isArray(items) && items.length > 0) {
                    let location = items[0];
                    
                    ui.addBubble(new H.ui.InfoBubble(location.position, {
                        content: location.title
                    }));
                    
                    $('#address').val(location.title);
                }
                else {
                    ConcreteAlert.error({
                        title: <?php echo json_encode(t('Geocoding error')); ?>,
                        message: <?php echo json_encode(t('No address found')); ?>,
                    });
                }
            }, (error) => {
                ConcreteAlert.error({
                    title: <?php echo json_encode(t('Geocoding failure')); ?>,
                    message: error
                });
            });
        }

        function loadMap (position) {
            map = new H.Map(
                document.getElementById('map'),
                defaultLayers.vector.normal.map,
                {
                    zoom: 12,
                    center: position,
                    pixelRatio: window.devicePixelRatio || 1
                }
            );
            
            window.addEventListener('resize', () => map.getViewPort().resize());
            
            behavior = new H.mapevents.Behavior(new H.mapevents.MapEvents(map));
            
            ui = H.ui.UI.createDefault(map, defaultLayers, language);
            ui.removeControl('mapsettings');
            
            let mapSettings = new H.ui.MapSettingsControl({
                baseLayers: [{
                    label: 'Map View', layer: defaultLayers.vector.normal.map
                }, {
                    label: 'Satellite', layer: defaultLayers.raster.satellite.map
                }, {
                    label: 'Terrain', layer: defaultLayers.raster.terrain.map
                }]
            });
            
            ui.addControl('customized', mapSettings);
            
            let zoom = ui.getControl('zoom');
            let scalebar = ui.getControl('scalebar');

            mapSettings.setAlignment('top-right');
            zoom.setAlignment('top-right');
            scalebar.setAlignment('bottom-right');
            
            setUpClickListener(map);
            addInfoBubble(map, position);
            
            function setUpClickListener(map) {
                map.addEventListener('tap', function (e) {
                    let target = e.target;
                    
                    if (!(target instanceof H.map.Marker)) {
                        let position = map.screenToGeo(e.currentPointer.viewportX, e.currentPointer.viewportY);
                        
                        marker.setGeometry(position);
                        moveMap(position);
                        geocodeReverse(position);
                    }
                });
            }

            function addInfoBubble(map, position) {
                let name = $.trim($('#name').val());
                let group = new H.map.Group();
                map.addObject(group);
                
                group.addEventListener('tap', function (e) {
                    let bubble = new H.ui.InfoBubble(e.target.getGeometry(), {
                        content: e.target.getData()
                    });
                    
                    ui.addBubble(bubble);
                }, false);
                
                addDraggableMarker(group, behavior, position, name);
            }
            
            function addDraggableMarker(group, behavior, position, data) {
                marker = new H.map.Marker(position, {
                    icon: icon,
                    volatility: true 
                });
                
                marker.draggable = true;
                marker.setData(data);
                group.addObject(marker);

                map.addEventListener('dragstart', function(e) {
                    let target = e.target;
                    let pointer = e.currentPointer;
                    
                    if (target instanceof H.map.Marker) {
                        let position = map.geoToScreen(target.getGeometry());
                        target['offset'] = new H.math.Point(pointer.viewportX - position.x, pointer.viewportY - position.y);
                        behavior.disable();
                    }
                }, false);
                
                map.addEventListener('dragend', function(e) {
                    let target = e.target;
                    
                    if (target instanceof H.map.Marker) {
                        behavior.enable();
                        let position = target.getGeometry();
                        moveMap(position);
                        geocodeReverse(position);
                    }
                }, false);
                
                map.addEventListener('drag', function(e) {
                    let target = e.target,
                    pointer = e.currentPointer;
                    
                    if (target instanceof H.map.Marker) {
                        let position = map.screenToGeo(pointer.viewportX - target['offset'].x, pointer.viewportY - target['offset'].y);
                        target.setGeometry(position);
                    }
                }, false);
            }
        }
        
        function setMapPosition(position) {
            map.setCenter(position, true);
            marker.setGeometry(position);
        }
        
        function moveMap(position) {
            let bubbles = ui.getBubbles();
            
            bubbles.forEach((bubble) => {
                bubble.close();
            });
            
            map.setCenter(position, true);
                
            latitude.val(position.lat.toFixed(4));
            longitude.val(position.lng.toFixed(4));
        }
    <?php } ?>
});
</script>
