/**
 * HERE Map package for Concrete CMS
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Alex Borisov <linuxoidoz@gmail.com>
 * @copyright 2021, Alex Borisov. All Rights Reserved.
 * @package Concrete\Package\ab_here_map
 */

$(document).ready(function(e) {
    if (CCM_EDIT_MODE) {
        return;
    }
    
    $.each($('.ab-here-map[data-here-map]'), function(index, element) {
        var data = JSON.parse($(element).attr('data-here-map'));
        initMap(element, data);
    });

    function initMap(element, data) {
        let api_key = data.api_key;
        
        if (api_key === '') {
            return;
        }
        
        let bid = data.bid;
        let lang = data.lang;
        let lat = data.lat;
        let lng = data.lng;
        let pin = data.pin;
        let zoom = data.zoom;
        let lock_wheel = data.lock_wheel;
        
        let position = new H.geo.Point(lat, lng);
        
        let platform = new H.service.Platform({
            'apikey': api_key
        });
        
        let defaultLayers = platform.createDefaultLayers({ pois: true, lg: lang.substring(0, 2) });
        
        let icon = new H.map.Icon(pin, {
            size: { w: 40, h: 40 },
        });
        
        let map = new H.Map(
            document.getElementById('ab_here_map_' + bid),
            defaultLayers.vector.normal.map,
            {
                zoom: zoom,
                center: position,
                pixelRatio: window.devicePixelRatio || 1
            }
        );
        
        window.addEventListener('resize', () => map.getViewPort().resize());
        let behavior = new H.mapevents.Behavior(new H.mapevents.MapEvents(map));
        
        if (lock_wheel) {
            behavior.disable(H.mapevents.Behavior.WHEELZOOM);
        }
        
        let ui = H.ui.UI.createDefault(map, defaultLayers, lang);
        ui.removeControl('mapsettings');
        
        let zoom_ctrl = ui.getControl('zoom');
        let scale_ctrl = ui.getControl('scalebar');
        
        zoom_ctrl.setAlignment('top-right');
        scale_ctrl.setAlignment('bottom-right');
        
        addInfoBubble(map, position);
        
        map.addEventListener('tap', onMapClick);
        
        function addInfoBubble(map, position) {
            let group = new H.map.Group();
            map.addObject(group);
            
            group.addEventListener('tap', function (e) {
                let bubble = new H.ui.InfoBubble(e.target.getGeometry(), {
                    content: e.target.getData()
                });
                
                ui.addBubble(bubble);
            }, false);
            
            addMarker(group, behavior, position);
        }
            
        function addMarker(group, behavior, position) {
            let marker = new H.map.Marker(position, {
                icon: icon,
            });
            
            let popup = '<div class="map-popup">';
            
            if (data.name !== '') {
                popup += '<h5>' + data.name + '</h5>';
            }
            
            if (data.address !== '') {
                popup += '<div class="clearfix"><i class="fas fa-home me-2 float-start"></i>' + data.address + '</div>';
            }
            
            if (data.phone !== '') {
                popup += '<div class="clearfix"><i class="fas fa-phone me-2 float-start"></i>' + data.phone + '</div>';
            }
            
            if (data.email !== '') {
                popup += '<div class="clearfix"><i class="fas fa-envelope me-2 float-start"></i>' + data.email + '</div>';
            }
            
            popup += '<div class="my-2 clearfix">';
            
            if (data.pic !== '') {
                popup += '<div class="float-start me-2 mb-1">' + data.pic + '</div>';
            }
            
            if (data.desc !== '') {
                popup += '<div class="desc">' + data.desc + '</div>';
            }
            
            popup += '</div>';
            
            popup += '</div>';
            
            marker.setData(popup);
            group.addObject(marker);
        }
        
        function onMapClick(e) {
            let target = e.target;
            
            if (!(target instanceof H.map.Marker)) {
                let bubbles = ui.getBubbles();
                                
                bubbles.forEach((bubble) => {
                    bubble.close();
                });
            }
        }
    }
});
