<?php

/**
 * HERE Map package for Concrete CMS
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * @author Alex Borisov <linuxoidoz@gmail.com>
 * @copyright 2021, Alex Borisov. All Rights Reserved.
 * @package Concrete\Package\ab_here_map
 */
 
namespace Concrete\Package\AbHereMap;

use Concrete\Core\Package\Package;
use Concrete\Core\Block\BlockType\BlockType;
use Concrete\Core\Asset\Asset;
use Concrete\Core\Asset\AssetList;

class Controller extends Package {

	protected $pkgHandle = 'ab_here_map';
	protected $appVersionRequired = '9.0.0';
	protected $pkgVersion = '1.0.0';
	protected $pkgAutoloaderMapCoreExtensions = true;
	
    public function getPackageDescription() 
	{
		return t('HERE Map package');
	}
	
	public function getPackageName() 
	{
		return t('HERE Map');
	}
	
	public function install() 
	{
		$pkg = parent::install();
		BlockType::installBlockType('ab_here_map', $pkg);
	}
    
    public function on_start()
    {
        $al = AssetList::getInstance();
        
        $al->register('css', 'here_maps_ui', 'https://js.api.here.com/v3/3.1/mapsjs-ui.css', ['version' => '3.1', 'local' => false, 'position' => Asset::ASSET_POSITION_HEADER], $this);
        $al->register('javascript', 'here_maps_core', 'https://js.api.here.com/v3/3.1/mapsjs-core.js', ['version' => '3.1', 'local' => false, 'position' => Asset::ASSET_POSITION_HEADER], $this);
        $al->register('javascript', 'here_maps_service', 'https://js.api.here.com/v3/3.1/mapsjs-service.js', ['version' => '3.1', 'local' => false, 'position' => Asset::ASSET_POSITION_HEADER], $this);
        $al->register('javascript', 'here_maps_events', 'https://js.api.here.com/v3/3.1/mapsjs-mapevents.js', ['version' => '3.1', 'local' => false, 'position' => Asset::ASSET_POSITION_HEADER], $this);
        $al->register('javascript', 'here_maps_ui', 'https://js.api.here.com/v3/3.1/mapsjs-ui.js', ['version' => '3.1', 'local' => false, 'position' => Asset::ASSET_POSITION_HEADER], $this);
        
        $al->registerGroup('here_maps', [
                ['css', 'here_maps_ui'],
                ['javascript', 'here_maps_core'],
                ['javascript', 'here_maps_service'],
                ['javascript', 'here_maps_events'],
                ['javascript', 'here_maps_ui'],
            ]
        );
    }

}
